import { useRef, useState, useCallback, useEffect } from 'react';

function useLatest(value) {
  var ref = useRef(value);
  ref.current = value;
  return ref;
}
function isRefObject(x) {
  return x && typeof x === 'object' && 'current' in x;
}
function hasSupport() {
  return typeof window !== 'undefined' && 'IntersectionObserver' in window && 'IntersectionObserverEntry' in window && 'intersectionRatio' in IntersectionObserverEntry.prototype;
}

function useElementInView(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$ref = _ref.ref,
      forwardedRef = _ref$ref === void 0 ? null : _ref$ref,
      _ref$root = _ref.root,
      root = _ref$root === void 0 ? null : _ref$root,
      _ref$rootMargin = _ref.rootMargin,
      rootMargin = _ref$rootMargin === void 0 ? '0px' : _ref$rootMargin,
      _ref$threshold = _ref.threshold,
      threshold = _ref$threshold === void 0 ? 0 : _ref$threshold,
      _ref$defaultInView = _ref.defaultInView,
      defaultInView = _ref$defaultInView === void 0 ? false : _ref$defaultInView,
      _ref$disconnectOnceVi = _ref.disconnectOnceVisible,
      disconnectOnceVisible = _ref$disconnectOnceVi === void 0 ? false : _ref$disconnectOnceVi,
      onChange = _ref.onChange;

  var _useState = useState({
    entry: undefined,
    elementInView: defaultInView
  }),
      observerEntry = _useState[0],
      setObserverEntry = _useState[1]; // Store our Intersection Observer instance to a ref for its lifecycle.


  var observerInstanceRef = useRef(null); // Element refs. We store the previously tracked element to ensure we only update when the element has changed
  // Along with the element supplied via the callback ref.

  var callbackElementRef = useRef(null);
  var prevTrackedElementRef = useRef(null); // Helper refs.

  var isObservingRef = useRef(false);
  var isMountedRef = useRef(true);
  var onChangeRef = useLatest(onChange);
  var observeElement = useCallback(function (node) {
    if (isObservingRef.current || !observerInstanceRef.current) {
      return;
    }

    isObservingRef.current = true;
    observerInstanceRef.current.observe(node);
  }, []);
  var disconnect = useCallback(function () {
    if (!isObservingRef.current || !observerInstanceRef.current) {
      return;
    }

    isObservingRef.current = false;
    observerInstanceRef.current.disconnect();
  }, []); // Instantiates the Intersection Observer.
  // It will determine the element based off how it was provided
  // and return a warning if no element was found via the options to assign one.

  var registerObserver = useCallback(function () {
    if (!hasSupport()) {
      return;
    }

    var element = null;

    if (callbackElementRef.current) {
      element = callbackElementRef.current;
    } else if (forwardedRef instanceof HTMLElement) {
      element = forwardedRef;
    } else if (isRefObject(forwardedRef)) {
      element = forwardedRef.current;
    } // Don't update or recall the register function unless the element has changed.


    if (prevTrackedElementRef.current === element) return;
    prevTrackedElementRef.current = element;

    if (!element) {
      // eslint-disable-next-line
      console.warn('🚨 No element has been found, are you sure you correctly assigned a ref?');
      return;
    } // Ensure we only create the instance once


    if (!observerInstanceRef.current) {
      var instance = new IntersectionObserver(function (_ref2) {
        var entry = _ref2[0];
        var isIntersecting = entry.isIntersecting && instance.thresholds.some(function (threshold) {
          return entry.intersectionRatio >= threshold;
        }); // Disconnect the instance once the observed entry is in view, and option to disconnectOnceVisible has been set

        if (isIntersecting && disconnectOnceVisible) {
          disconnect();
        }

        if (onChangeRef.current) {
          onChangeRef.current(entry);
        } // Ensure we are still mounted before attempting to set state.
        // TODO: This may not be needed for a callback that isn't called too often.


        if (isMountedRef.current) {
          setObserverEntry({
            entry: entry,
            elementInView: isIntersecting
          });
        }
      }, {
        root: root,
        rootMargin: rootMargin,
        threshold: threshold
      });
      observerInstanceRef.current = instance;
    }

    disconnect();
    observeElement(element);
  }, [root, rootMargin, threshold, disconnectOnceVisible, observeElement, disconnect, forwardedRef, onChangeRef]);
  var callbackRef = useCallback(function (node) {
    if (node) {
      callbackElementRef.current = node;
      registerObserver();
    }
  }, [registerObserver]);
  useEffect(function () {
    if (forwardedRef) {
      registerObserver();
    }

    return function () {
      isMountedRef.current = false;
      disconnect();
    };
  }, [forwardedRef, registerObserver, disconnect]);
  return {
    entry: observerEntry.entry,
    inView: observerEntry.elementInView,
    assignRef: callbackRef,
    disconnect: disconnect
  };
}

export { useElementInView };
//# sourceMappingURL=use-element-in-view.esm.js.map
